@@ -68,7 +68,7 @@
 #' nano$socket
 #' nano$listener[[1]]
 #'
-#' nano$setopt("ms", "send-timeout", 1000)
+#' nano$setopt("send-timeout", 1000)
 #'
 #' nano$listen(url = "inproc://nanonextgen")
 #' nano$listener
@@ -99,12 +99,12 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
     r <- dial(socket, url = dial, autostart = autostart)
     if (r == 0L) {
       nano[["dialer"]] <- attr(socket, "dialer")
-      nano[["dialer_setopt"]] <- function(type = c("bool", "int", "ms", "size",
-                                                   "string", "uint64"),
-                                          opt,
+      nano[["dialer_getopt"]] <- function(opt) lapply(.subset2(nano, "dialer"),
+                                                      getopt,
+                                                      opt = opt)
+      nano[["dialer_setopt"]] <- function(opt,
                                           value) invisible(lapply(.subset2(nano, "dialer"),
                                                                   setopt,
-                                                                  type = type,
                                                                   opt = opt,
                                                                   value = value))
       if (!autostart) nano[["dialer_start"]] <- function(async = TRUE) {
@@ -119,12 +119,12 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
     r <- listen(socket, url = listen, autostart = autostart)
     if (r == 0L) {
       nano[["listener"]] <- attr(socket, "listener")
-      nano[["listener_setopt"]] <- function(type = c("bool", "int", "ms", "size",
-                                                     "string", "uint64"),
-                                            opt,
+      nano[["listener_getopt"]] <- function(opt) lapply(.subset2(nano, "listener"),
+                                                        getopt,
+                                                        opt = opt)
+      nano[["listener_setopt"]] <- function(opt,
                                             value) invisible(lapply(.subset2(nano, "listener"),
                                                                     setopt,
-                                                                    type = type,
                                                                     opt = opt,
                                                                     value = value))
       if (!autostart) nano[["listener_start"]] <- function() {
@@ -141,12 +141,12 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
     r <- dial(socket, url = url, autostart = autostart)
     if (r == 0L) {
       nano[["dialer"]] <- attr(socket, "dialer")
-      nano[["dialer_setopt"]] <- function(type = c("bool", "int", "ms", "size",
-                                                   "string", "uint64"),
-                                          opt,
+      nano[["dialer_getopt"]] <- function(opt) lapply(.subset2(nano, "dialer"),
+                                                      getopt,
+                                                      opt = opt)
+      nano[["dialer_setopt"]] <- function(opt,
                                           value) invisible(lapply(.subset2(nano, "dialer"),
                                                                   setopt,
-                                                                  type = type,
                                                                   opt = opt,
                                                                   value = value))
       if (!autostart) nano[["dialer_start"]] <- function(async = TRUE) {
@@ -162,12 +162,12 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
     r <- listen(socket, url = url, autostart = autostart)
     if (r == 0L) {
       nano[["listener"]] <- attr(socket, "listener")
-      nano[["listener_setopt"]] <- function(type = c("bool", "int", "ms", "size",
-                                                     "string", "uint64"),
-                                            opt,
+      nano[["listener_getopt"]] <- function(opt) lapply(.subset2(nano, "listener"),
+                                                        getopt,
+                                                        opt = opt)
+      nano[["listener_setopt"]] <- function(opt,
                                             value) invisible(lapply(.subset2(nano, "listener"),
                                                                     setopt,
-                                                                    type = type,
                                                                     opt = opt,
                                                                     value = value))
       if (!autostart) nano[["listener_start"]] <- function() {
@@ -178,6 +178,8 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
     }
     invisible(r)
   }
+  nano[["getopt"]] <- function(opt) getopt(socket,
+                                           opt = opt)
   nano[["recv"]] <- function(mode = c("serial", "character", "complex", "double",
                                       "integer", "logical", "numeric", "raw"),
                              block = NULL,
@@ -204,11 +206,8 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
                                                           data = data,
                                                           mode = mode,
                                                           timeout = timeout)
-  nano[["setopt"]] <- function(type = c("bool", "int", "ms", "size",
-                                               "string", "uint64"),
-                               opt,
+  nano[["setopt"]] <- function(opt,
                                value) setopt(socket,
-                                             type = type,
                                              opt = opt,
                                              value = value)
 
