@@ -103,9 +103,8 @@ SEXP rnng_random(SEXP n) {
 
   PROTECT(vec = Rf_allocVector(REALSXP, vlen));
   double *pvec = REAL(vec);
-  for (R_xlen_t i = 0; i < vlen; i++) {
+  for (R_xlen_t i = 0; i < vlen; i++)
     pvec[i] = (double) nng_random();
-  }
 
   UNPROTECT(1);
   return vec;
@@ -133,6 +132,7 @@ SEXP rnng_device(SEXP s1, SEXP s2) {
 SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP follow, SEXP method, SEXP headers,
                 SEXP data, SEXP response, SEXP pem) {
 
+  const int conv = LOGICAL(convert)[0];
   nng_url *url;
   nng_http_client *client;
   nng_http_req *req;
@@ -140,7 +140,7 @@ SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP follow, SEXP method, SEXP headers,
   nng_aio *aio;
   nng_tls_config *cfg = NULL;
   int xc;
-  uint16_t code;
+  uint16_t code, relo;
 
   if ((xc = nng_url_parse(&url, CHAR(STRING_ELT(http, 0)))))
     goto exitlevel1;
@@ -216,13 +216,14 @@ SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP follow, SEXP method, SEXP headers,
     nng_tls_config_free(cfg);
   nng_aio_free(aio);
 
-  code = nng_http_res_get_status(res);
+  code = nng_http_res_get_status(res), relo = code >= 300 && code < 400 ? 1 : 0;
 
-  if (code >= 300 && code < 400 && LOGICAL(follow)[0])
-    return rnng_ncurl(Rf_mkString(nng_http_res_get_header(res, "Location")),
-                      convert, follow, method, headers, data, response, pem);
+  if (relo && LOGICAL(follow)[0]) {
+    SET_STRING_ELT(nano_addRedirect, 0, Rf_mkChar(nng_http_res_get_header(res, "Location")));
+    return rnng_ncurl(nano_addRedirect, convert, follow, method, headers, data, response, pem);
+  }
 
-  SEXP out, vec, cvec = R_NilValue, rvec = R_NilValue;
+  SEXP out, vec, cvec, rvec;
   void *dat;
   size_t sz;
   const char *names[] = {"status", "headers", "raw", "data", ""};
@@ -231,6 +232,22 @@ SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP follow, SEXP method, SEXP headers,
 
   SET_VECTOR_ELT(out, 0, Rf_ScalarInteger(code));
 
+  if (relo) {
+    const R_xlen_t rlen = Rf_xlength(response);
+    switch (TYPEOF(response)) {
+    case STRSXP:
+      PROTECT(response = Rf_lengthgets(response, rlen + 1));
+      SET_STRING_ELT(response, rlen, Rf_mkChar("Location"));
+      break;
+    case VECSXP:
+      PROTECT(response = Rf_lengthgets(response, rlen + 1));
+      SET_VECTOR_ELT(response, rlen, Rf_mkString("Location"));
+      break;
+    default:
+      PROTECT(response = Rf_mkString("Location"));
+    }
+  }
+
   if (response != R_NilValue) {
     const R_xlen_t rlen = Rf_xlength(response);
     PROTECT(rvec = Rf_allocVector(VECSXP, rlen));
@@ -256,21 +273,24 @@ SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP follow, SEXP method, SEXP headers,
       break;
     }
     UNPROTECT(1);
+  } else {
+    rvec = R_NilValue;
   }
   SET_VECTOR_ELT(out, 1, rvec);
+  if (relo) UNPROTECT(1);
 
   nng_http_res_get_data(res, &dat, &sz);
   vec = Rf_allocVector(RAWSXP, sz);
-  memcpy(RAW(vec), dat, sz);
+  if (dat != NULL)
+    memcpy(RAW(vec), dat, sz);
   SET_VECTOR_ELT(out, 2, vec);
 
-  if (code >= 300 && code < 400) {
-    cvec = Rf_mkString(nng_http_res_get_header(res, "Location"));
-  } else if (LOGICAL(convert)[0]) {
-    SEXP expr;
-    PROTECT(expr = Rf_lang2(nano_RtcSymbol, vec));
-    cvec = R_tryEvalSilent(expr, R_BaseEnv, &xc);
+  if (conv) {
+    PROTECT(cvec = Rf_lang2(nano_RtcSymbol, vec));
+    cvec = R_tryEvalSilent(cvec, R_BaseEnv, &xc);
     UNPROTECT(1);
+  } else {
+    cvec = R_NilValue;
   }
   SET_VECTOR_ELT(out, 3, cvec);
 
